﻿using Models;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Windows.Forms;

namespace FastReport.NET_Sample_001
{
    public partial class BookList : Form
    {
        public BookList()
        {
            InitializeComponent();
            dgv.DataSource = GetList(); // Listeyi yüklüyoruz.
        }

        /// <summary>
        /// Sadece seçili satırları rapora yükler ve raporu gösterir.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ReportSelectedRows_Click(object sender, EventArgs e)
        {
            DataTable dt = GetFilledDataTableForReport(dgv);
            if (dt != null)
                CreateReport(dt);
        }

        /// <summary>
        /// Bütün satırları rapora yükler ve raporu gösterir.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ReportAllRows_Click(object sender, EventArgs e)
        {
            DataTable dt = GetFilledDataTableForReport(dgv, false);
            if (dt != null)
                CreateReport(dt);
        }


        /// <summary>
        /// Veritabanından kitap listesini çeker ve geriye döndürür.
        /// </summary>
        /// <returns></returns>
        List<BookInfoModel> GetList()
        {
            using (DBDataContext db = new DBDataContext())
            {
                return db.Books.OrderBy(x => x.Type.Name).ThenBy(x => x.Writer.Name).Select(x => new BookInfoModel() { Type = x.Type.Name, Writer = x.Writer.Name, Book = x.Name }).ToList();
            }
        }

        /// <summary>
        /// Rapor olarak sunulacak olan bilgileri DataTable'a yükler ve bu DataTable'ı geriye döndürür.
        /// </summary>
        /// <param name="dgv"></param>
        /// <param name="justSelectedRows">Sadece seçili satırların yüklenip yüklenmeyeceğini belirler.</param>
        /// <returns></returns>
        DataTable GetFilledDataTableForReport(DataGridView dgv, bool justSelectedRows = true)
        {
            if (justSelectedRows && dgv.SelectedRows.Count < 1) // Sadece seçili satırlar talep edildiyse ve hiç satır seçilmemişse
            {
                MessageBox.Show("Seçili satır bulunamadığı için işlem iptal edildi.", "Geçersiz İşlem", MessageBoxButtons.OK, MessageBoxIcon.Hand);
                return null;
            }

            DataTable dt = new DataTable();
            dt.Columns.AddRange(new DataColumn[] 
            { 
                new DataColumn("Book"),
                new DataColumn("Type"),
                new DataColumn("Writer") 
            });

            List<DataGridViewRow> rc;

            if (justSelectedRows)
                rc = dgv.SelectedRows.Cast<DataGridViewRow>().ToList();
            else
                rc = dgv.Rows.Cast<DataGridViewRow>().ToList();

            for (int i = 0; i < rc.Count; i++)
            {
                dt.Rows.Add(
                    rc[i].Cells[0].Value.ToString(),
                    rc[i].Cells[1].Value.ToString(),
                    rc[i].Cells[2].Value.ToString()
                );
            }

            return dt;
        }

        /// <summary>
        /// Raporu oluşturur.
        /// </summary>
        /// <param name="data">Rapora yüklenecek olan veri</param>
        private void CreateReport(DataTable data)
        {
            Report report = new Report(); // Rapor dosyasını oluşturuyoruz.

            report.Load("Report_Sample.frx"); // Programla aynı dizinden şablon olarak kullandığımız raporu yüklüyoruz.

            // Gelen verileri şablon olarak oluşturduğumuz rapor içerisinde ismi belirtilen tabloya register ediyoruz.
            report.RegisterData(data, "BooksWithTypes");

            if (report.Prepare())       // Raporu hazırlıyoruz.
            {
                report.ShowPrepared();  // Hazırlanmış olan raporun gösterilmesini sağlıyoruz.
            }
        }
    }
}
